;-----------------------------------------------------------------------------
;  Program file..: chgrattr.asm 
;  Author........: Paul Bosselaers (some stuff from Rick Whitt's Clock.asm)
;  Date..........: 5/13/95
;  Used by.......: Clipper 5.x
;  Function......: changes attribute at a specified screen row between
;                  specified cols using direct screen writes.
;                  returns NIL
;
;                  Syntax of the call is as follows:
;
;                  CHGROWATTR( srow, scol, ecol, attr )
; 
;                         srow  = row coordinate for clock from 0 to 24
;                         scol  = starting col from 0 to 79
;                         ecol  = ending col from 0 to 79
;                         attr = attributes for display. The rules
;                                for calculating the attribute figures are
;                                detailed below.
;
;                  The attribute number is a one byte number that is used
;                  to set foreground and background colors, and also 
;                  intensity.
;                    
;                  For Monochrome:   Normal ......................   7
;                                    Intense .....................  15
;                                    Normal underlined ...........   1
;                                    Intense underlined ..........   9
;                                    Reverse ..................... 112
;                                    Blinking normal ............. 135
;                                    Blinking intense ............ 143        
;                                    Blinking normal underlined .. 129 
;                                    Blinking intense underlined . 137
;                                    Blinking reverse ............ 240
;
;                  -----------------------------------------------------
;
;                  For color:        Foreground          Background
;                                    ----------          ----------
;
;                     Black              0                    0
;                     Blue               1                   16
;                     Green              2                   32
;                     Cyan               3                   48
;                     Red                4                   64
;                     Magenta            5                   80
;                     Brown              6                   96
;                     White              7                  112
;
;                     Gray               8
;                     Light blue         9
;                     Light green       10
;                     Light cyan        11                  Not
;                     Light red         12               Available
;                     Light magenta     13
;                     Yellow            14
;                     Bright white      15
;
;
;                     To get the color attribute, add the numbers together
;                     of the foreground and background colors that you
;                     want. Add 128 to this to make it blink. For example:
;
;                     YELLOW on RED =  14 + 64  =  78
;                     
;                     To make the above blinking, add 128 + 78 = 206
;
;
;----------------------------------------------------------------------------
.model large
.286

;-----------------------------------------------------------;
; Converts row, col and video page to an offset in video    ;
; memory. (Assumes 80 columns/row)                          ;
; leaves result in DI                                       ;
;-----------------------------------------------------------;

GetV_Offset     MACRO row, col, vpage

                IFB     <row>
                  mov   dh,00h
                ELSE
                  mov     dh, row               ; load registers
                ENDIF

                IFB    <col>
                  mov   dl,00h
                ELSE
                  mov     dl, col
                ENDIF

                IFB     <vpage>
                  mov   bl,00h
                ELSE
                  mov     bl,vpage              ; page #
                ENDIF
                xor     bh,bh                   ; zero BH
                
                mov     al,160                  ; row * 160
                mul     dh                      ; result in AX
                shl     dl,1                    ; col * 2
                xor     dh,dh                   ; byte to word in DX
                add     ax,dx                   ; add the two
                mov     di,ax                   ; save result in DI
                mov     ax,1000h                ; length of 1 video page
                mul     bx                      ; page * 1000
                add     di,ax                   ; complete offset address
ENDM


public  CHGROWATTR
        
extrn   __parni:far
extrn   __parinfo:far
extrn   __ret:far

.data
sRow            db      ?                       ; display start coordinates
sCol            db      ?
numCols         dw      ?                       ; number of cols to change
VideoSeg        dw      0B800h                  ; video segment address
VideoOff        dw      0                       ; video offset
VideoPage       db      0                       ; video page

.code
CHGROWATTR      proc    far

                push    bp              ; save return address
                mov     bp,sp           ; address variable on stack
                push    ds              ; must save regs
                push    es
                push    si
                push    di

                mov     ah,15           ; get video mode and page number 
                int     10h                 
                mov     VideoPage,bh    ; save page number

                cmp     al,7                    ; mono ?
                jne     Color                   ; no, then jump
                mov     VideoSeg,0B000h         ; save address and 
                jmp     DoneWithDisplay

Color:
                mov     VideoSeg,0B800h

DoneWithDisplay:

                xor     ax,ax           ; get number of parameters
                push    ax
                call    __parinfo       ; call information function
                add     sp,2            ; fix up stack
                cmp     al,4
                jne     Mid_Jump

GetPara:
                mov     ax,1            ; get 1st parameter (sRow)
                push    ax
                call    __parni
                add     sp,2
                cmp     al,0
                jb      Mid_Jump        ; exit if not valid
                cmp     al,24
                ja      Mid_Jump
                mov     sRow,al         ; else save the row number

                mov     ax,2            ; get Starting Column number
                push    ax
                call    __parni
                add     sp,2
                cmp     al,0
                jb      Exit
                cmp     al,79
                ja      Exit
                mov     sCol,al
                jmp     short Part2

Mid_Jump:
                jmp     exit

Part2:
                GetV_Offset srow,scol,VideoPage         ; Get Video Offset
                inc     di                              ; move to attribute        
                mov     VideoOff,di

                mov     ax,3            ; get Ending Column number
                push    ax
                call    __parni
                add     sp,2
                cmp     al,0
                jb      Exit      
                cmp     al,79
                ja      Exit
                cmp     al,sCol         ; end Col must be >= start col
                jb      Exit

                mov     bl,sCol         ; calc number of columns
                xor     bh,bh
                sub     ax,bx           ; num cols = ecol - scol + 1
                inc     ax
                mov     numCols,ax
                
                mov     ax,4            ; get attribute
                push    ax
                call    __parni
                add     sp,2
                cmp     al,0
                jb      Exit      

WriteIt:
                mov     dx,VideoSeg
                mov     es,dx
                mov     di,VideoOff
                mov     cx,numCols

DoLoop:
                stosb
                inc     di
                loop    DoLoop

Exit:
                call     __ret          ; return NIL

                pop     di
                pop     si
                pop     es              ; restore registers
                pop     ds                      
                pop     bp
                ret                     ; far return to Clipper
                
CHGROWATTR      endp
                end


