/*
  Scrollable Multi-Page gets
  using Clipper's Get, TBrowse and TBcolumn classes
  OOP version, Classy 2.x required, full Classy2 syntax

Inspired from Ascroll.prg, procedural version of
  Jo W. French, dba Practical Computing , CIS ID: 74730, 1751

Optimized & converted to Classy 2.4 version (26 July 95)
  Daniel-Tri Nguyen, NK-Informatique, France, CIS: 100333,3347

Note:
  - I try my best to translate in English, please forgive for possible
    omission or clumsiness

  - The class can be easily inherited from standard TB class. Thus, could
    prevent to put some instance var in the Export portion. And avoiding to
    declare a oTB handle. However, I decide not to do that because the class
    is simple so that it is not necessary to weigh down it with all the
    features of standard TB

  - You may look more carefully and closely to OOP. It will change radically
    (in the GOOD sense) your way of thinking and programming.


How to call this class from client code:

1. Define (optionnaly) all the get captions inside a 1D array. If there is
   several gets on the same row, put all captions un the same line

2. Define all the gets. The row, col coordinates are ALL relatives to the TB
   ie: 1 = first data-displayable line of TB (under the heading separator)
   there is no limite for maximum row value
   collumn value are relative to the TB's left border

3. The TB acts like a window which reveal only a subset of the whole gets.
   the current visible gets is called GGV (Group Get Visible)
   Gets is achieved actually by extracting the GGV from global getlist
   and applying readmodal() on it

Example how to call ScrolGet objet (in the simpliest form,
without get captions):

  static function BlaBla
  local getlist := {} , mydata_, kk
    // data to gets and modify
    mydata_ := array(70)
    dispbegin()
    for kk := 1 to len(mydata_)
      mydata_[kk] := padr("Line" + str(kk, 3, 0), 50)
      @ kk, 2 get mydata_[kk]
    next
    // keep trace of relatives ccordinates, we'll need them to convert
    // in screen absolute coord. later
    // :colorspec : 1st = unactive gets, 2nd: get having focus
    // Note: DON'T forget this line
    aeval(getlist, {|o| o:cargo := { o:row, o:col } ,;
           o:colorspec := "n/w, w+/b" } )
    dispend()
    CLS
    ScrolGet():new(3, 5, 15, 60, getlist, nil,;
                   "Description", "n/bg,w+/b,w/n"):Exec()

TBget.RMK

  makepath[.prg] := "Your_Prg_Path"

  //---Blinker 2.x
  //.OBJ.EXE:
  //  Blinker OUTPUT G FILE TBgCli, TBget LIB D:\3rd\Classy\CLASSY
  //---CauseWay
  .OBJ.EXE:
    WL32 TBgCli TBget, G ,, D:\3rd\Classy\CLASSY D:\3rd\CauseWay\CWCLP52

  .PRG.OBJ:
     Clipper $< /n /m

  TBget.obj  : TBget.prg
  TBgCli.obj : TBgCli.prg
  G.EXE : TBgCli.obj  TBget.obj

*/

#include "D:\3rd\Classy\Classy2.ch"
#include "getExit.ch"
#include "Inkey.ch"



CREATE CLASS ScrolGet

  var ggv_    // Groupe Gets Visibles (GGV)

  var GetList // whole gets defined by client code

  method FocBotLef // positionn get focus on the leftmost of the last row
  method ExtracGgv // Extract GGV from global getlist


PROTECTED:

EXPORT:
  // must be visible by standard TB's methods
  var oTB     // internal TB handle
  var nTop1   // absolute coord. of the 1st data-displayable line in TB
  var nBotLim // TB's bottom limite, relative / ::nTop1
  var nPosTB  // current position of TB's cursor, relative / ::nTop1

  method  init
  method  Exec

END CLASS
//Classe: ScrolGet




/*
   Init classe

   nTop, nLeft, nBot, nRight : coord. of TB (compulsory)
      including the TB's border

   getlist : compulsory
      getlist already defined BEFORE instantiating an object.
      Recall: @ L,C get getlist[x]
      L, C are all relative to the TB borders

   capt_ : optionnal (dfaut = "")
      all the gets captions (define line / line)

   cHead : column heading

   cClr : TB color. Default : "w/b"

*/
method Init(nTop, nLeft, nBot, nRight, getlist, capt_, cHead, cClr)

local oCol, kk, nDiff


cClr := if(cClr == nil, "w/b", cClr)

::GetList := getlist

nBot := min(nBot, maxrow() -1)

// border of the TB (hope make it nicier)
@ nTop, nLeft, nBot -1, nRight box "Ŀ" color cClr
// Signal key's usages:
@ nBot, nLeft, nBot, nRight box space(1) color "w/n"
@ nBot, nLeft +1  say "Esc-End" color "w/n"
@ nBot, nLeft +12 say "Ctrl-" + chr(24) + chr(25) + ":Scroll" color "w/n"
@ nBot, nLeft +30 say "C-PgUp/PgDn: Top/Bottom" color "w/n"

::oTB := TbrowseNew(nTop +1, nLeft +1, nBot -2, nRight -1)
::oTB:colorSpec := cClr

// starting position TB's cursor
::nPosTB := 1

// How to draw captions inside the TB ?
// Calculate the maximum value of TB's cursor position
if valtype(capt_) == "A"
  // there's caption defined, display its content
  oCol := tbColumnNew("", {|| capt_[::nPosTB] })
  // maximum value of TB's cursor position is obviously the length of capt_[]
  ::nBotLim := len(capt_)
else
  // ---- No captions defined ----
  // with this, the heading is left justified
  //oCol := tbColumnNew("", {|| space(::oTB:nRight - ::oTB:nLeft -2) })
  // with this (slightly quicker), nut the heading is centered
  oCol := tbColumnNew("", {|| nil })

  // maximum value of TB's cursor position IS NOT simply len(getlist)
  // because there can be several gets on a same row
  // the only accurate way is to count each different row value
  ::nBotLim := 0
  nDiff := -1
  for kk := 1 to len(getlist)
    // Recall: before call, each get object cargo was initialized with:
    // ... { o:row, o:col }
    if nDiff != getlist[kk]:cargo[1]
      nDiff := getlist[kk]:cargo[1]
      ::nBotLim ++
    endif
  next
endif
// TEST
//@ 24,65 say "BottomLim =" + str(::nBotLim, 3, 0) color "w/rb"

if valtype(cHead) == "C"
  oCol:heading := cHead
  ::oTB:headsep := chr(196)
endif

// UNlight TB's cursor. The main actor here is get objects
::oTB:autolite := .F.

::oTB:goTopBlock    := {|| ::nPosTB := 1 }
::oTB:goBottomBlock := {|| ::nPosTB := ::nBotLim }

::oTB:skipBlock := {|nJump| - ::nPosTB ;
    + (::nPosTB := max(1, min(::nBotLim, ::nPosTB + nJump) ) ) }

::oTB:AddColumn(oCol)

// absolute coord. of the 1st data-displayable position in the TB
::oTB:ForceStable()
::nTop1 := row()


return (self)
//Init: ScrolGet




/*
  The execution module of this class
*/
method Exec

local nKey, sksav_, nFocus, oGet, lRexSav

// pointer for subarrays in sksav_[]
#define SK_NUMKEY  1
#define SK_bSAV    2

// special navigating keys terminate read + achieving navig. actions
// ATTN: recpect SK_... order
sksav_ := ;
{ ;
  { K_CTRL_DOWN , } ,;
  { K_CTRL_UP   , } ,;
  { K_CTRL_PGUP , } ,;
  { K_CTRL_PGDN , }  ;
}

for nKey := 1 to len(sksav_)
  sksav_[nKey, SK_bSAV] := ;
    setkey(sksav_[nKey, SK_NUMKEY], {|| getActive():exitState := GE_ESCAPE})
next

// starting position for TB's cursor and starting line of GGV
nFocus := 1
::nPosTB := 1

// extract the starting window of GGV
::ExtracGgv()

// allow Up/Down key to terminate a read
lRexSav := readExit(.T.)

nKey := 0
while nKey != K_ESC
  // TEST
  //@ 24,40 say "TBCurs =" + str(::nPosTB, 3, 0) color "w/b"
  oGet := ::ggv_[nFocus]
  if oGet:preBlock != NIL .and. ! eval(oGet:preblock, oGet)
    // This condition may be removed if 5.2's GetSys.prg has
    // been modified to correct the bug in the Settle function
    // re' entering ReadModal with a false WHEN condition.
    //
    if ++nFocus > len(::ggv_)
      nFocus :=1
    endif
    loop
  else
    readModal(::ggv_, nFocus)
  endif

  nKey := lastkey()
  do case
  case ! empty(ascan({ K_CTRL_DOWN, K_DOWN, K_ENTER }, nKey)) .and.;
       ::nPosTB < ::nBotLim
    while .T.
      // Down mouvment: force the TB's curs position to the last line of
      // visible window. Therefore, down() scroll immediately
      ::oTB:rowpos := ::oTB:rowCount
      ::oTB:down()
      ::ExtracGgv()
      // position get focus on the leftmost position of the current row
      nFocus := ::FocBotLef(::ggv_)

      // if the TB's cursor falls on a line without any gets
      // continue to down toward a next available get. Therefore,
      // exit only if the new line have a get of if TB's bottom is reached
      if atail(::ggv_):row == (::nTop1 + ::oTB:rowcount - 1) .or.;
        ::nPosTB == ::nBotLim
        exit
      endif
    enddo

  case ! empty(ascan({ K_CTRL_UP , K_UP }, nKey)) .and. ::nPosTB > 1
    while .T.
      // Up Mvt: force TB's curs position to the 1st line of visib. win
      // so that up() scrolls immediatly
      ::oTB:rowpos := 1
      ::oTB:up()
      ::ExtracGgv()
      nFocus := 1
      // if the TB's cursor falls on a line without any gets
      // continue to move upward to a next available get. Therefore,
      // exit only if the new line have a get of if TB's top is reached
      if ::ggv_[1]:row == ::nTop1 .or. ::nPosTB == 1
        exit
      endif
    enddo

  case nKey == K_CTRL_PGUP .and. ::nPosTB > 1
    ::oTB:gotop() // TB's topmost position
    ::ExtracGgv()
    nFocus := 1

  case nKey == K_CTRL_PGDN .and. ::nPosTB < ::nBotLim
    ::oTB:goBottom() // TB's Bottom most position
    ::ExtracGgv()
    nFocus := len(::ggv_)

  case ! empty(ascan({ K_CTRL_DOWN, K_DOWN, K_ENTER }, nKey)) .and.;
       ::nPosTB == ::nBotLim
    // position get focus on the leftmost positionof the current row
    nFocus := ::FocBotLef(::ggv_)

  case nKey == K_PGUP .and. ::nPosTB > 1
    ::oTB:rowpos := 1
    ::oTB:pageUp()
    ::ExtracGgv()
    nFocus := 1

  case nKey == K_PGDN .and. ::nPosTB < ::nBotLim
    ::oTB:rowpos := ::oTB:rowCount
    ::oTB:pageDown()
    ::ExtracGgv()
    nFocus := 1
  endcase
enddo

// restore savings
for nKey := 1 to len(sksav_)
  setkey(sksav_[nKey, SK_NUMKEY], sksav_[nKey, SK_bSAV])
next

readExit(lRexSav)

return (self)
//Exec






/*
  The down mvt reveals a new line of get. Position the focus on this row
  but if there is several gets on this row, put focus on the leftmost one
*/
method FocBotLef

local nElem, nLastLine

nLastLine := atail(::ggv_):row

nElem := len(::ggv_)
while ::ggv_[nElem]:row == nLastLine
  nElem --
enddo
// we've just passed it, come back on first on the row
nElem ++

return nElem
//FocBotLef




/*
   Extract the GGV from getlist
*/
method ExtracGgv

local kk, nLig, n1stLine

::ggv_ := {}

// IMPORTANT: TB should be stabilized
::oTB:ForceStable()

//=== Determine the row N (relative to TB) which marks the beginnig of GGV
//
// Le scrolling of GGVs is only achived at each page. Each time, the TB'cursor
//   position is just under the GGV. The difference is then always
//   oTB:rowcount lines from the 1st line of GGV
// Except only one case: when GGV and TB cursor position are both at the
//   beginning of the TB. In this case, all row N are forced to 1

// Nof GGV's 1st line (relative to TB)
n1stLine := max(1, ::nPosTB - ::oTB:rowpos + 1)

for kk := 1 to len( ::GetList )
  // Recall: before instantiating the ScrolGet objet, you should save the
  // relatives coord. of each get in cargo instance var. in the fowlowing form
  // aeval(getlist, {|o| o:cargo := { o:row, o:col },;
  //                     o:colorspec := "n/w, w+/b" } )
  nLig := ::GetList[kk]:cargo[1]
  // is the row N is included in the GGV range ?
  if nLig >= n1stLine .and. nLig <= n1stLine + ::oTb:rowcount - 1
    aadd(::ggv_, ::GetList[kk] )
    // tanslate row, col to screen absolute coordinate
    // in order to apply readmodal() on them
    atail(::ggv_):row := nLig - n1stLine + ::nTop1
    atail(::ggv_):col := ::oTB:nLeft + ::GetList[kk]:cargo[2]
  endif
next
dispbegin()
aeval( ::ggv_, {|o| o:display()} )
dispend()
// TEST
//@ 24,0  say "1st GGV's Line =" + str(n1stLine, 3, 0) color "w+/g"

return (self)
//ExtracGgv







//TBget.prg//

