#include "inkey.ch"
#include "panel.ch"

MEMVAR GetList

* Simple program convert a DBF to a Btrieve file
* Uses UPLOAD function shown below.
*
PROC main
LOCAL aData := ARRAY(6)
AFILL(aData,SPACE(10))
CLEAR SCREEN
IF !N_XLOGIN()
	? "Btrieve is not available"
	RETURN
ENDIF
SET COLOR TO "GR+/B"
SET SCORE OFF
SET MESSAGE TO 24 CENTER NONE
SET CONFIRM ON
CLEAR SCREEN
@ 0,0 TITLE "RaSQL/B Upload Utility"
@ 3,0 SAY { "DBF", ;
	"Index 1", ;
	"Index 2", ;
	"Index 3", ;
	"Index 4", ;
	"Btrieve File" } ;
	EDIT FROM aData
IF LASTKEY() != K_ESC .AND. !EMPTY(aData[1]) .AND. !EMPTY(aData[6])
	USE (aData[1]) INDEX (aData[2]),(aData[3]),(aData[4]),(aData[5])
	IF BUPLOAD(aData[6])
		N_XGOTOTOP()
		@ 5,5 DRAW BOX SIZE 18,70
		QBROWSE(6,6,22,74)
		N_XCLOSE()
	ENDIF
ENDIF
@ 23,0 SAY ""
N_XLOGOUT()
RETURN

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* This module creates a new Btrieve table from the currently open 
* DBF and associated index files.  The parsing routine 
* shown below can handle most of the standard index expressions.
* 
* To use this module, USE a DBF and the associated index files that 
* you want made into permanent indexes.  The file must be in the 
* current directory and it must be opened without an ALIAS.  Then 
* call UPLOAD with the name of the new Btrieve file to be created.
*
* After completion of UPLOAD, the DBF will no longer be open.
* 
* In addition to the Btrieve file, an RQB structure file will also
* be created.  The name of the RQB file will be the same as the
* Btrieve file, with the entension ".RQB".  See the manual about
* RQB files.
*
* For example, to upload NETDEMO.DBF and its associated indexes 
* into a Btrieve table named NETDEMO.DAT and a structure file
* NETDEMO.RQB:
* 
*           N_XLOGIN()
*           USE NETDEMO index NETDEMO1, NETDEMO2
*           BUPLOAD("NETDEMO.DAT")
* 
* NETDEMO.DBF and its associated index files are included on the 
* RaSQL disk.
*
* Requires that BEXTEND.PRG be compiled and linked in, too.
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* 
Function BUPLOAD(cFile)
local nCurrent,cStrFile,cExpr,cWorkFile,k, cStruct, cRQB
IF (!used())					// is DBF open?
	? "No file is in use"
	return .F.
endif
nCurrent = select()				// save current area
cStruct = N_XDSTRUCT()
cStrFile = 'A'+LTRIM(STR(SECONDS()))		// temp structure file name
cWorkFile = 'B'+LTRIM(STR(SECONDS()))		// temp structure file name
N_XMAKEDBF(cStrFile,2)				// create temp structure file
copy structure extended to (cWorkFile)		// create DBF structure file
use (cStrFile) alias _UPLOAD_ new
append from (cWorkFile)				// convert to RaSQL structure
select (nCurrent)
k = 1
do while !empty(indexkey(k))			// add index clauses
	cexpr = indexkey(k)
	select _upload_
	IDX2DBF(cExpr)
	select (nCurrent)
	k = k + 1
enddo
select _UPLOAD_
use
if N_XMAKETBL("",cFile,cStrFile)		// create Btrieve file
	if '.' $ cFile
		cRQB = left(cFile,at('.',cFile)) + "RQB"	// RQB file
	else
		cRQB = trim(cFile) + ".RQB"
	endif
	MEMOWRIT(cRQB,cStruct)			// write structure
	select (nCurrent)
	IF N_XUSE(cFile,cStruct)		// open empty Btrieve file
		N_XUPLOAD("*",.T.)		// uploda data positionally
	endif
endif
select (nCurrent)
ERASE (cStrFile)
ERASE (cWorkFile)
return (N_XERROR() <= 0)

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* This nifty little module originated with Dave Wohlferd at 
* Pinnacle.  The purpose of it is to process an index expression 
* and create output suitable for use with N_XMAKETBL or N_XMAKENDX.
* 
* IDX2DBF will add the index entries to the structure file used by 
* N_XMAKETBL or N_XMAKENDX.   The file is assumed to be opened in 
* the current area.
* 
* The parameter is the index expression.
* 
* Most simple index expressions are processed.  Feel free to 
* improve on it.  If you come up with more comprehensive functions, 
* let us know and we'll put your name in lights, too!
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
function IDX2DBF(cExpr)
local lUpper,lDes,nPosit,cFieldName
local nRetVal := 0
do while !empty(cExpr)
	 lUpper = .f.
	 lDes = .f.
	 nPosit = at("+",cExpr)
	 nPosit = iif(nPosit = 0, len(cExpr)+1, nPosit)
	 cFieldName = upper(alltrim(substr(cExpr,1,nPosit-1)))
	 if substr(cFieldName,1,4) == "DESC"
			lDes = .t.
			cFieldName = ltrim(substr(cFieldName,8))
			cFieldName = substr(cFieldName,2,len(cFieldName)-2)
	 endif
	 if substr(cFieldName,1,4) == "UPPE"
			lUpper = .t.
			cFieldName = ltrim(substr(cFieldName,6))
			cFieldName = substr(cFieldName,2,len(cFieldName)-2)
	 endif
	 if substr(cFieldName,1,3) == "STR"
			cFieldName = ltrim(substr(cFieldName,4))
			cFieldName = substr(cFieldName,2,len(cFieldName)-2)
			if "," $ cFieldName
				 cFieldName = substr(cFieldName,1,at(",",cFieldName)-1)
			endif
	 endif
	 if "(" $ cFieldName
			? "Unable to convert expression: "+cFieldName
			cExpr = ""
			nRetVal = 0
	 else
			nRetVal = nRetVal + 1
			append blank
			replace Field_name with cFieldName, Field_type with "X", More with .t., ;
				 Uppercase with lUpper, Descending with lDes
			cExpr = iif(nPosit < len(cExpr),substr(cExpr,nPosit+1),"")
	 endif
enddo
replace More with .f.
return (nRetVal)
